"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataImageView = exports.DataImageModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const jupyter_dataserializers_1 = require("jupyter-dataserializers");
const version_1 = require("./version");
const ndarray = require("ndarray");
/**
 * Model for the the data widgets based image widget.
 */
class DataImageModel extends base_1.DOMWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), {
            _model_name: DataImageModel.model_module,
            _model_module_version: DataImageModel.model_module_version,
            _model_module: DataImageModel.model_module,
            _view_name: DataImageModel.view_name,
            _view_module: DataImageModel.view_module,
            _view_module_version: DataImageModel.view_module_version,
            data: ndarray([]),
        });
    }
}
exports.DataImageModel = DataImageModel;
DataImageModel.serializers = Object.assign(Object.assign({}, base_1.DOMWidgetModel.serializers), { data: jupyter_dataserializers_1.data_union_serialization });
DataImageModel.model_name = 'DataImageModel';
DataImageModel.model_module = 'jupyter-datawidgets';
DataImageModel.model_module_version = version_1.version;
DataImageModel.view_name = 'DataImageView';
DataImageModel.view_module = 'jupyter-datawidgets';
DataImageModel.view_module_version = version_1.version;
/**
 * View for the the data widgets based image widget.
 */
class DataImageView extends base_1.DOMWidgetView {
    constructor() {
        super(...arguments);
        this.canvas = null;
    }
    initialize(parameters) {
        super.initialize(parameters);
        (0, jupyter_dataserializers_1.listenToUnion)(this.model, 'data', this.update.bind(this), true);
    }
    /**
     * Called when view is rendered.
     */
    render() {
        super.render();
        this.pWidget.addClass('jupyter-widgets');
        this.pWidget.addClass('widget-image');
        this.update(); // Set defaults.
    }
    /**
     * Update the contents of this view
     *
     * Called when the model is changed.  The model may have been
     * changed by another view or by a state update from the back-end.
     */
    update() {
        let data = (0, jupyter_dataserializers_1.getArray)(this.model.get('data'));
        let url;
        if (data === null || data.data.length === 0) {
            url = '';
            this.el.setAttribute('width', '0');
            this.el.setAttribute('height', '0');
        }
        else {
            if (data.shape.length !== 3 || data.shape[2] !== 4) {
                throw new Error(`DataImage data has invalid shape: ${JSON.stringify(data.shape)}`);
            }
            if (this.canvas === null) {
                this.canvas = document.createElement('canvas');
            }
            this.el.setAttribute('width', `${data.shape[0]}`);
            this.el.setAttribute('height', `${data.shape[1]}`);
            this.canvas.setAttribute('width', `${data.shape[0]}`);
            this.canvas.setAttribute('height', `${data.shape[1]}`);
            const ctx = this.canvas.getContext('2d');
            const imageData = new ImageData(new Uint8ClampedArray(data.data), data.shape[0], data.shape[1]);
            ctx.putImageData(imageData, 0, 0);
            url = this.canvas.toDataURL();
        }
        this.el.src = url;
        return super.update();
    }
    remove() {
        if (this.canvas !== null) {
            this.canvas = null;
        }
        super.remove();
    }
    // While we support ipywidget 7, the below TS ignore must be used.
    // Once we only support ipywidgets 8 and above, replace with preinitalize
    /**
     * The default tag name.
     *
     * #### Notes
     * This is a read-only attribute.
     */
    // @ts-ignore: 2611
    get tagName() {
        // We can't make this an attribute with a default value
        // since it would be set after it is needed in the
        // constructor.
        return 'img';
    }
}
exports.DataImageView = DataImageView;
//# sourceMappingURL=media.js.map